Imports System.ComponentModel.DataAnnotations
Imports System.Data.Entity
Imports System.Data.Entity.ModelConfiguration
Imports Model

Namespace DataAccess
  Public Class BreakAwayContextFluent
	  Inherits DbContext

	Public Property Destinations As DbSet(Of Destination)

	Public Property Lodgings As DbSet(Of Lodging)

	Public Property Trips As DbSet(Of Trip)

	Public Property People As DbSet(Of Person)

	Public Property PersonPhotos As DbSet(Of PersonPhoto)

	Protected Overrides Sub OnModelCreating(ByVal modelBuilder As DbModelBuilder)
	  modelBuilder.Entity(Of Lodging)().Map(Sub(m) m.ToTable("Lodging")).Map<Resort>(Function(m)
		  m.ToTable("Resorts")
		  m.MapInheritedProperties()
	 )

	  '  modelBuilder.Entity<Resort>().Map(m=>
	  ' {
	  '  m.ToTable("Resorts");
	  '  m.MapInheritedProperties();
	  '});
	  modelBuilder.Configurations.Add(New DestinationConfiguration)
	 ' modelBuilder.Configurations.Add(new LodgingConfiguration());
	  modelBuilder.Configurations.Add(New PersonConfiguration)
	  modelBuilder.Configurations.Add(New TripConfiguration)
	  modelBuilder.Configurations.Add(New AddressConfiguration)
	  modelBuilder.Configurations.Add(New PersonPhotoConfiguration)
	  modelBuilder.Entity(Of Person)().Property(Function(p) p.Address.StreetAddress).HasColumnName("StreetAddress")
	  modelBuilder.ComplexType(Of Address)().Property(Function(p) p.StreetAddress).HasColumnName("StreetAddress")
	  modelBuilder.ComplexType(Of Address)()
	  modelBuilder.Entity(Of InternetSpecial)().HasRequired(Function(s) s.Accommodation).WithMany(Function(l) l.InternetSpecials).HasForeignKey(Function(s) s.AccommodationId)
	  modelBuilder.Entity(Of Destination)().HasMany(Function(d) d.Lodgings).WithRequired(Function(l) l.Destination)
	  modelBuilder.Entity(Of Person)().ToTable("People")
	  modelBuilder.Entity(Of PersonPhoto)().ToTable("People")
	  ' modelBuilder.Entity<Resort>().ToTable("Resorts");

	  'modelBuilder.Entity<Lodging>().Map(m =>
	  '{
	  '  m.ToTable("Lodging");
	  '}).Map<Resort>(m =>
	  '{
	  '  m.ToTable("Resorts");
	  '});

	  '//modelBuilder.Entity<Destination>().Property(d => d.Name).IsRequired;
	  'modelBuilder.Entity<Destination>().Property(d => d.Description).HasMaxLength(500);
	  'modelBuilder.Entity<Destination>().Property(d => d.Photo).HasColumnType("image");
	  'modelBuilder.Entity<Trip>().HasKey(t => t.Identifier).Property(t => t.Identifier).HasDatabaseGeneratedOption(
	  '  DatabaseGeneratedOption.Identity);
	  'modelBuilder.Entity<Person>().HasKey(p => p.SocialSecurityNumber);
	  'modelBuilder.Entity<Person>().Property(p => p.SocialSecurityNumber).HasDatabaseGeneratedOption(
	  '  DatabaseGeneratedOption.None);
	  'modelBuilder.Entity<Person>().Property(p => p.RowVersion).IsRowVersion;
	  'modelBuilder.Entity<Lodging>().Property(l => l.Name).IsRequired.HasMaxLength(200);

	  'modelBuilder.Entity<Lodging>()
	  '    .Property(l => l.Owner)
	  '    .IsRequired()
	  '    .HasMaxLength(200);

	  'modelBuilder.Entity<Destination>()
	  '    .Property(d => d.Description)
	  '    .HasColumnType("text");

	  'modelBuilder.Entity<Destination>()
	  '    .Property(d => d.Country)
	  '    .IsConcurrencyToken();
	End Sub
  End Class

  Public Class AddressConfiguration
	  Inherits ComplexTypeConfiguration(Of Address)

	Public Sub New()
	  Property(Function(a) a.State).HasMaxLength(30)

	  Property(Function(a) a.StreetAddress).HasColumnName("StreetAddress")
	End Sub
  End Class

  Public Class PersonConfiguration
	  Inherits EntityTypeConfiguration(Of Person)

	Public Sub New()
'      
'        HasKey(p => p.SocialSecurityNumber);
'        Property(p => p.SocialSecurityNumber).HasDatabaseGeneratedOption(
'        DatabaseGeneratedOption.None);
'        Property(p => p.RowVersion).IsRowVersion;
'       
	  Property(Function(p) p.SocialSecurityNumber).IsConcurrencyToken

	  Property(Function(p) p.Address.StreetAddress).HasColumnName("StreetAddress")
	  ' HasRequired(p => p.Photo).WithRequiredPrincipal(pp=>pp.PhotoOf).WillCascadeOnDelete(true);
	  ToTable("People") 'added along when mapping multiple entites to one table
	End Sub
  End Class

  Public Class TripConfiguration
	  Inherits EntityTypeConfiguration(Of Trip)

	Public Sub New()
	  HasKey(Function(t) t.Identifier).Property(Function(t) t.Identifier).HasDatabaseGeneratedOption(DatabaseGeneratedOption.Identity)

	  Property(Function(p) p.RowVersion).IsRowVersion
	End Sub
  End Class

  Public Class PersonPhotoConfiguration
	  Inherits EntityTypeConfiguration(Of PersonPhoto)

	Public Sub New()
	  Me.HasKey(Function(p) p.PersonId)
	  HasRequired(Function(p) p.PhotoOf).WithRequiredDependent(Function(p) p.Photo)
	  ToTable("People")
	End Sub
  End Class

  Public Class DestinationConfiguration
	  Inherits EntityTypeConfiguration(Of Destination)

	Public Sub New()
	  Me.Property(Function(d) d.Name).IsRequired.HasColumnName("LocationName")

	  Property(Function(d) d.DestinationId).HasColumnName("LocationID")

	  Property(Function(d) d.Description).HasMaxLength(500)

	  Property(Function(d) d.Photo).HasColumnType("image")
	  Ignore(Function(d) d.TodayForecast)
	  ' ToTable("Locations");

	  Map(Sub(m)
		  m.Properties(Function(d) New With {Key d.Name, Key d.Country, Key d.Description})
		  m.ToTable("Locations")
	  End Sub)
	  Map(Sub(m)
		  m.Properties(Function(d) New With {Key d.Photo})
		  m.ToTable("LocationPhotos")
	  End Sub)
	End Sub
  End Class

  Public Class LodgingConfiguration
	  Inherits EntityTypeConfiguration(Of Lodging)

	Public Sub New()
	  'Map(m =>
	  '{
	  '  m.ToTable("Lodgings");
	  '})
	  '   .Map<Resort>(m =>
	  '   {
	  '     m.ToTable("Resorts");
	  '     m.MapInheritedProperties();
	  '   });
	  Property(Function(l) l.Name).IsRequired.HasMaxLength(200)

	  Property(Function(l) l.Owner).IsUnicode(False)

	  Property(Function(l) l.MilesFromNearestAirport).HasPrecision(8, 1)
	  HasOptional(Function(l) l.PrimaryContact).WithMany(Function(p) p.PrimaryContactFor)
	  HasOptional(Function(l) l.SecondaryContact).WithMany(Function(p) p.SecondaryContactFor)

	  '   HasRequired(l => l.Destination).WithMany(d => d.Lodgings).WillCascadeOnDelete(false);

	  'Map(m =>
	  '    {
	  '        m.ToTable("Lodging");
	  '        m.Requires("LodgingType").HasValue("Standard");
	  '    })
	  '.Map<Resort>(m =>
	  '    {
	  '        m.Requires("LodgingType").HasValue("Resort");
	  '    });

	  'Map(m =>
	  '{
	  '  m.ToTable("Lodging");
	  '  m.Requires("IsResort").HasValue(false);
	  '})
	  '.Map<Resort>(m =>
	  '{
	  '  m.Requires("IsResort").HasValue(true);
	  '});
	End Sub
  End Class
End Namespace
